/**=========================================================
 * Module: calendar-ui.js
 * This script handle the calendar demo with draggable 
 * events and events creations
 =========================================================*/
App.controller('CalendarController', ['$rootScope', '$scope', '$modal', '$http', 'periodoService', '$timeout', 'APP_SETTINGS', function ($rootScope, $scope, $modal, $http, periodoService, $timeout, app_settings) {
    'use strict';

    if (!$.fn.fullCalendar) return;

    $rootScope.PeriodoActual = periodoService.getPeriodo();

    function initCalendar(calElement) {

        $rootScope.EscuProfesional = "";

        var EstudianteID = window.localStorage.getItem("EstudianteID");
        calElement.fullCalendar({
            isRTL: $scope.app.layout.isRTL,
            height: '1000',
            defaultView: 'agendaWeek',
            allDaySlot: false,
            columnFormat: {
            week: 'ddd D/M',
            day: 'dddd D/M'
            },
            titleFormat: {
                week: "MMM D",
                day: "MMM D",
            },
            header: {
                left: 'prev,next today',
                center: 'title',
                right: 'agendaWeek,agendaDay'
            },
            buttonIcons: { // note the space at the beginning
                prev: ' fa fa-caret-left',
                next: ' fa fa-caret-right'
            },
            buttonText: {
                today: 'hoy',
                month: 'mes',
                week: 'semana',
                day: 'dia'
            },
            editable: false,
            droppable: false,
            events: {
                url: app_settings.serviceUrl('/Horario/ObtenerHorario?EstudianteId=' + EstudianteID),
                type: 'GET',
                success: function (data) {
                    console.log(data);
                    $rootScope.EscuProfesional = data[0].Especialidad;
                    $rootScope.Programa = data[0].Programa;

                    //console.log("Escuela Profesional: " + $rootScope.EscuProfesional);
                    //console.log($rootScope.Programa);
                },
                error: function () {
                    $modal.open({
                        templateUrl: '/myModalError.html',
                        controller: ModalInstanceErrorCtrl,
                        resolve: {
                            data: function () {
                                return "Error al obtener los datos del horario";
                            }
                        },
                        size: 'sm'
                    });
                },
            },
            eventClick: function (calEvent, jsEvent, view) {
                //console.log(calEvent);
                var AulaId = calEvent.AulaId;
                var HorarioDetalleId = calEvent.HorarioDetalleId;
                $http.get(app_settings.serviceUrl("/horario/ObtenerHorarioDetalle"), {
                    params: {
                        AulaId: AulaId,
                        HorarioDetalleId: HorarioDetalleId
                    }
                }).success(function (data) {
                    //console.log(data);
                    $modal.open({
                        templateUrl: '/myModalContent.html',
                        controller: ModalInstanceCtrl,
                        size: 'sm',
                        resolve: {
                            data: function () {
                                return data;
                            }
                        }
                    });

                }).error(function () {
                    $modal.open({
                        templateUrl: '/myModalError.html',
                        controller: ModalInstanceErrorCtrl,
                        resolver: {
                            data: function () {
                                return "Sucedió un error";
                            }
                        },
                        size: 'sm'
                    });
                    //alert("Sucedió un error");
                })
            },
        });
    }


    $(function () {
        var calendar = $('#calendar');
        initCalendar(calendar);

    });
    var ModalInstanceCtrl = function ($scope, $modalInstance, data) {

        $modalInstance.opened.then(function () {

            $rootScope.Hora = data[0].Hora
            $rootScope.Asignatura = data[0].Asignatura;
            $rootScope.Sede = data[0].Sede;
            $rootScope.Ciclo = data[0].Ciclo;
            $rootScope.TS = data[0].Turno;
            $rootScope.Aula = data[0].Aula;
            $rootScope.Profesor = data[0].Profesor;

            $timeout(function () {
                $scope.initGmap = true;
            });
        });
        $scope.ok = function () {
            $modalInstance.close('closed');
        };
        $scope.cancel = function () {
            $modalInstance.dismiss('cancel');
        };
    };

    var ModalInstanceErrorCtrl = function ($scope, $modalInstance, data) {
        $modalInstance.opened.then(function () {

            $rootScope.MensajeErrorModal = data;
        });

        $scope.cancel = function () {
            $modalInstance.dismiss('cancel');
        };
    };
}]);